using System;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Tcp;

namespace ListScan
{
	/// <summary>
	/// Main executor.
	/// </summary>
	class Exe
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main(string[] args)
		{
			if (args.Length != 3)
			{
				Console.WriteLine("ListScan: <scanserver address> <zone list file> <XML scanning configuration file>");
				Console.WriteLine("Zone list file syntax: ");
				Console.WriteLine("   Plate line    : <brick>,<plate>,<map init string>");
                Console.WriteLine("   Zone line     : <id>,<minx>,<maxx>,<miny>,<maxy>,<output name>");
                Console.WriteLine("   MC output line: MC,<output name>");
                Console.WriteLine("   MC line       : MC,<id>,<x>,<y>,<sx>,<sy>,<postol>,<slopetol>");
                Console.WriteLine("");
                Console.WriteLine("   Note: first line must be a Plate line ");
				return;
			}
            // Store CultureInfo for the current culture. Note that the original culture
            // can be set and retrieved on the current thread object.
            System.Threading.Thread thread = System.Threading.Thread.CurrentThread;
            System.Globalization.CultureInfo originalCulture = thread.CurrentCulture;

            // Set the culture to en-US.
            thread.CurrentCulture = new System.Globalization.CultureInfo("en-US");


			System.IO.StreamReader r = new System.IO.StreamReader(args[2]);
			string config = r.ReadToEnd();
			r.Close();

			System.Runtime.Remoting.Channels.ChannelServices.RegisterChannel( new TcpChannel());
			SySal.DAQSystem.ScanServer ScanSrv = (SySal.DAQSystem.ScanServer)System.Runtime.Remoting.RemotingServices.Connect(typeof(SySal.DAQSystem.ScanServer), "tcp://" + args[0] + ":" + ((int)SySal.DAQSystem.OperaPort.ScanServer).ToString() + "/ScanServer.rem");
			ScanSrv.SetScanLayout(config);

            SySal.DAQSystem.Scanning.MountPlateDesc plated = new SySal.DAQSystem.Scanning.MountPlateDesc();
            
            System.IO.StreamWriter sw = null; // output file for manual check
            
            r = new System.IO.StreamReader(args[1]);
            string line = r.ReadLine() ;
            string [] tokens = line.Split(',');

            if ( tokens.Length != 3 )
            {
					Console.WriteLine("First Line must be a Plate line" );
					return ;
            }

            do
            {
                Console.WriteLine("process:"+line);

                tokens = line.Split(',');
                try
                {
                    if (tokens.Length == 1)
                    {
                        // COMMENT -> SKIP 
                        // if a manual check output file is open, write plate description
                        if (sw != null)
                            sw.WriteLine();
                            sw.WriteLine(line);
                    } 
                    else if (tokens.Length == 3)
                    {
                        // LOAD PLATE
                        plated.BrickId = Convert.ToInt32(tokens[0]);
                        plated.PlateId = Convert.ToInt32(tokens[1]);
                        plated.MapInitString = tokens[2];
                        plated.TextDesc = "";

                        Console.WriteLine("Load Plate: " + plated.PlateId);
                        if (!ScanSrv.LoadPlate(plated))
                        {
                            Console.WriteLine("ERROR: Unable to LOAD PLATE");
                            return;
                        }

                        // if a manual check output file is open, write plate description
                        if (sw != null)
                            sw.WriteLine(line);
                    }
                    else if (tokens.Length == 6)
                    {
                        // SCAN ZONE
                        int id;
                        double minx, maxx, miny, maxy;
                        id = Convert.ToInt32(tokens[0]);
                        minx = Convert.ToDouble(tokens[1]);
                        maxx = Convert.ToDouble(tokens[2]);
                        miny = Convert.ToDouble(tokens[3]);
                        maxy = Convert.ToDouble(tokens[4]);
                        string outname = tokens[5] ;

                        Console.WriteLine("  scan zone: " + id + " ( " + minx + ", " + maxx + ", " + miny + ", " + maxx + ", " + minx + ") " + outname);
                        SySal.DAQSystem.Scanning.ZoneDesc zd = new SySal.DAQSystem.Scanning.ZoneDesc();
                        zd.Series = id;
                        zd.MinX = minx;
                        zd.MaxX = maxx;
                        zd.MinY = miny;
                        zd.MaxY = maxy;
                        zd.Outname = outname ;
                        ScanSrv.Scan(zd);
                    }
                    else if (tokens.Length == 2)
                    {
                        // OPEN A FILE FOR MANUAL CHECK
                        //tokens[0] == "MC" ignored
                        string outname = tokens[1];

                        Console.WriteLine("open a file for manual check output: " + outname);
                        sw = new System.IO.StreamWriter(outname, true);
                        sw.WriteLine();
                        sw.WriteLine("START A NEW MANUAL CHECK SESSION"); //todo passare il trial
                        sw.WriteLine(plated.BrickId.ToString() + "," + plated.PlateId.ToString() + "," + plated.MapInitString);
                    }
                    else if (tokens.Length == 8)
                    {
                        // MANUAL CHECK
                        int id;
                        double x, y, sx, sy, postol, slopetol;
                        //tokens[0] == "MC" ignored
                        id       = Convert.ToInt32(tokens[1]);
                        x        = Convert.ToDouble(tokens[2]);
                        y        = Convert.ToDouble(tokens[3]);
                        sx       = Convert.ToDouble(tokens[4]);
                        sy       = Convert.ToDouble(tokens[5]);
                        postol   = Convert.ToDouble(tokens[6]);
                        slopetol = Convert.ToDouble(tokens[7]);

                        Console.WriteLine(" manual check: " + id + " ( " + x + ", " + y + ", " + sx + ", " + sy + ") pos tol: " 
                            + postol + " slope tol: " + slopetol );
                        SySal.DAQSystem.Scanning.ZoneDesc zd = new SySal.DAQSystem.Scanning.ZoneDesc();

                        SySal.DAQSystem.Scanning.ManualCheck.InputBaseTrack bt =
                            new SySal.DAQSystem.Scanning.ManualCheck.InputBaseTrack() ;
                        
                        bt.Id = id ;
                        bt.Position.X = x  ;
                        bt.Position.Y = y  ;
                        bt.Slope.X    = sx ;
                        bt.Slope.Y    = sy ;
                        bt.PositionTolerance = postol ;
                        bt.SlopeTolerance = slopetol;

                        SySal.DAQSystem.Scanning.ManualCheck.OutputBaseTrack outbt =
                            ScanSrv.RequireManualCheck(bt) ;

                        // write results
                        sw.Write("{0,8} {1,8} {2,8}          {3,8} {4,8}"
                            , bt.Id.ToString() 
                            , bt.Position.X.ToString("F1")
                            , bt.Position.Y.ToString("F1")
                            , bt.Slope.X.ToString("F4")
                            , bt.Slope.Y.ToString("F4")
                        );

                        if (!outbt.CheckDone)
                        {
                            sw.Write("     check not done\r\n");
                        }
                        else
                        {
                            if (!outbt.Found)
                                sw.Write("     track not found\r\n");
                            else
                                sw.Write("     found "
                                       + " " + outbt.Position.X.ToString("F1")
                                       + " " + outbt.Position.Y.ToString("F1")
                                       + " " + outbt.Slope.X.ToString("F4")
                                       + " " + outbt.Slope.Y.ToString("F4")
                                       //+ " " + outbt.Id.ToString("F4")
                                       + "\r\n" );
                        }
                    } 
                    else                       
                    {
                        Console.WriteLine("Skipping line: " + line);
                        continue;
                    }

                }
                catch (Exception x)
                {
                    Console.WriteLine("Error \n" + x.Message + "\nat line: " + line);
                    Console.WriteLine("Line skipped");
                }
            } while ((line = r.ReadLine()) != null && line.Length > 0);

            if (sw!=null) sw.Close();

			Console.WriteLine("List scan complete");
		}

        private static object Exception(string p)
        {
            throw new Exception("The method or operation is not implemented.");
        }
	}
}

