
#pragma once

#ifndef _CSBRICKCONNECT_
#define _CSBRICKCONNECT_
#include <vector>
//___________________________________________________________________________//


class TConfiguration
{
public:
   TConfiguration()  { SetDefaults() ; };
   virtual ~TConfiguration()  {};

   const char* ShowPar() ;
   void SetDefaults() ;
   int ReadConfig(const char* file) ;

#  define MAXSTRLEN 256 

   char path            [MAXSTRLEN] ;
   char cs2file         [MAXSTRLEN] ;
   char csspot_markset  [MAXSTRLEN] ;
   char lateral_markset [MAXSTRLEN] ;
   char workdir         [MAXSTRLEN] ;
   char rawdir          [MAXSTRLEN] ;
   char tlgdir          [MAXSTRLEN] ;
   char scanip          [MAXSTRLEN] ;
   char scanconf        [MAXSTRLEN] ;
   char listscanpath    [MAXSTRLEN] ;
   char batchlinkpath   [MAXSTRLEN] ;
   char batchlinkconfig [MAXSTRLEN] ;
   char tlgselpath      [MAXSTRLEN] ;
   char tlg2txtpath     [MAXSTRLEN] ;
   char foundpath       [MAXSTRLEN] ;
   char mcresultspath   [MAXSTRLEN] ;
   char mcscanconf      [MAXSTRLEN] ;
   char mergeresults    [MAXSTRLEN] ;
};

//___________________________________________________________________________//
struct zone_t
{
   struct { double min; double  max ;} X ;
   struct { double min; double  max ;} Y ;
};
//___________________________________________________________________________//
struct segment_t
{
   int    ID ;
   double X, Y, Z ;
   double SX, SY ;
   int    NGrains ;
   double sigma;
};
//___________________________________________________________________________//
class TPrediction
{
public:
   TPrediction() ;
   TPrediction( int id, double x, double y, double z, double sx, double sy, int ngrains ) ;
   virtual ~TPrediction()  ;

   void Set( int id, double x, double y, double z, double sx, double sy, int ngrains ) ;
   void SetID( int id ) ;
   void SetXYZ( double x, double y, double z) ;
   void SetSXSY( double sx, double sy) ;
   void SetNGrains( int ngrains ) ;
   void EvalScanZone( double size_x, double size_y ) ;
   const char* SelectionString( double res_pos, double res_ang, double increase_with_slope) ;
   
   zone_t GetScanZone() ;

   int  ReadCandidates(const char* fname) throw (...)  ;
   void AddCandidate( segment_t seg ) ;
   void AddCandidate(const char* line ) throw (...) ;
   int  NCandidates()                 ;
   void ClearCandidates(void)              ;
   segment_t GetCandidate( int i )    ;


   const char* Print() ;
   const char* PrintCandidate( int i ) ;
   const char* PrintCandidateForManualCheck( int i )  ;
   const char* PrintCandidateResiduals( int i ) ;
   const char* PrintScanZone() ;
   const char* PrintScanZone_forListScan( const char* folder, int plate_number , int trial_number) ;
   void ProjectTo( double z ) ;

   const char* FileName               ( const char* folder, int plate_number, int trial_number, const char* ext ) ;
   const char* RawDataFileName        ( const char* folder, int plate_number, int trial_number ) ;
   const char* TlgDataFileName        ( const char* folder, int plate_number, int trial_number ) ;
   const char* TlgSelectedDataFileName( const char* folder, int plate_number, int trial_number ) ;
   const char* TxtSelectedDataFileName( const char* folder, int plate_number, int trial_number ) ;

protected:
   segment_t fP ; // prediction
   std::vector<segment_t> fF ; // found
   zone_t fScanZone ;

};
//_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _//
inline TPrediction::TPrediction() {} ;
inline TPrediction::~TPrediction() { ClearCandidates() ;} ;
inline TPrediction::TPrediction( int id, double x, double y, double z, double sx, double sy, int ngrains ) 
            { Set(id,x,y,z,sx,sy,ngrains) ; } 

inline void TPrediction::Set( int id, double x, double y, double z, double sx, double sy, int ngrains ) 
   { SetID(id); SetXYZ(x,y,z); SetSXSY(sx, sy); SetNGrains(ngrains); } 
inline void TPrediction::SetID     ( int id )                      { fP.ID=id; }
inline void TPrediction::SetXYZ    ( double x, double y, double z) { fP.X=x; fP.Y=y; fP.Z=z; }
inline void TPrediction::SetSXSY   ( double sx, double sy)         { fP.SX=sx; fP.SY=sy; }
inline void TPrediction::SetNGrains( int ngrains )                 { fP.NGrains = ngrains ; } 

inline zone_t TPrediction::GetScanZone() { return fScanZone ; }

inline void TPrediction::AddCandidate( segment_t seg )  { fF.push_back( seg ); };
inline int  TPrediction::NCandidates()                    { return (int) fF.size() ; } 
inline void TPrediction::ClearCandidates(void)            { fF.clear() ; }
inline segment_t TPrediction::GetCandidate( int i ) throw(...)   
                 { if (i<0||i>=NCandidates()) throw "GetCandidate out of range"; return fF[i] ; };
//___________________________________________________________________________//

class TPredictionSet
{
public:
   TPredictionSet() ;
   virtual ~TPredictionSet() ;

   void AddSegment( TPrediction seg ) ;
   void AddSegment( const char* line ) throw (...) ;

   TPrediction* GetPrediction( int i ) throw(...) ;
   int N() ;

   int  ReadPredictions( const char* fname ) throw (...) ; 
   void EvaluateScanZones (double size_x, double size_y) ;
   void CreateZoneListFile( const char* fname, int brick_number, int plate_number, 
                            const char* map_fname, const char* rawfolder, 
                            int trial_number)  throw (...);

   bool IsNewTrial( const char* file, int trial ) ;

   const char* Print() ;
   const char* PrintScanZones() ;
   void ProjectTo( double z) ;

   const char* PrintCandidates() ;
   void SaveCandidates( const char* fname, int trial_number ) throw (...) ;
   const char* CandidatesFileName( const char* folder, int brick_number, int plate_number ) ;
   
   const char* PrintCandidatesForManualCheck( const char* results_fname , int brick_number, int plate_number, float plate_z, int trial_number, const char* map_fname , double pos_tol, double slope_tol) ;
   void CreateZoneListFileForManualCheck( const char* zonelist_fname, const char* results_fname , int brick_number, int plate_number, float plate_z, int trial_number, const char* map_fname , double pos_tol, double slope_tol) throw (...) ;
   const char* CandidatesFileNameForManualCheck( const char* folder, int brick_number, int plate_number ) ;

   const char* MergeResults( int brick_number, const char* outputfname, const char* mcresultspath ) ;

protected:
   std::vector<TPrediction> fSegments    ;

public:
   void Clear();
};
//_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _//
inline     TPredictionSet:: TPredictionSet() {};
inline     TPredictionSet::~TPredictionSet() { Clear() ; };

inline void TPredictionSet::AddSegment( TPrediction seg )  { fSegments.push_back( seg ); };
inline int  TPredictionSet::N()                    { return (int) fSegments.size() ; } 
inline void TPredictionSet::Clear(void)            { fSegments.clear() ; }
inline TPrediction* TPredictionSet::GetPrediction( int i ) throw(...)   
                 { if (i<0||i>=N()) throw "GetPrediction out of range"; return &fSegments[i] ; };

#endif
