using System;
using System.IO;

using SySal.Scanning.Plate.IO.OPERA;
using SySal.Tracking;

namespace tlg2txt
{
    class Program
    {
        static int Main(string[] args)
        {
            if (args.Length != 2)
            {
                Console.WriteLine("usage: tlg2txt <input.tlg> <output.txt>");
                return -1;
            }

            string path   = args[0];
            string output = args[1];

            // Store CultureInfo for the current culture. Note that the original culture
            // can be set and retrieved on the current thread object.
            System.Threading.Thread thread = System.Threading.Thread.CurrentThread;
            System.Globalization.CultureInfo originalCulture = thread.CurrentCulture;

            // Set the culture to en-US.
            thread.CurrentCulture = new System.Globalization.CultureInfo("en-US");

            
            FileStream r = null;
            StreamWriter w = null;

            try
            {
                Console.WriteLine("open : " + path );
                r = new FileStream(path, FileMode.Open, FileAccess.Read);
                LinkedZone lz = new LinkedZone(r);
                r.Close();

                Console.WriteLine( "number of tracks: " + lz.Length.ToString() );

                Console.WriteLine("write to : " + output);
                w = new StreamWriter(output);
                for (int i=0; i<lz.Length; i++) {
                    MIPEmulsionTrackInfo tr  = lz[i].Info;
                    MIPEmulsionTrackInfo top = lz[i].Top.Info   ;
                    MIPEmulsionTrackInfo bot = lz[i].Bottom.Info ;
                    string str = i.ToString()
                               + " " + tr.Intercept.X.ToString("F1")
                               + " " + tr.Intercept.Y.ToString("F1")
                               + " " + tr.Slope.X.ToString("F4")
                               + " " + tr.Slope.Y.ToString("F4")
                               + " " + tr.Count.ToString()
                               + " " + tr.Sigma.ToString("F3")
                               + " " + top.Intercept.X.ToString("F1")
                               + " " + top.Intercept.Y.ToString("F1")
                               + " " + top.Slope.X.ToString("F4")
                               + " " + top.Slope.Y.ToString("F4")
                               + " " + top.Count.ToString()
                               + " " + top.Sigma.ToString("F3")
                               + " " + bot.Intercept.X.ToString("F1")
                               + " " + bot.Intercept.Y.ToString("F1")
                               + " " + bot.Slope.X.ToString("F4")
                               + " " + bot.Slope.Y.ToString("F4")
                               + " " + bot.Count.ToString()
                               + " " + bot.Sigma.ToString("F3")
                               ;
                    Console.WriteLine(str);
                    w.WriteLine(str);
                }
                
                w.Close();
                return lz.Length;
            }
            catch (Exception x)
            {
                if (r != null)
                    r.Close();
                if (w != null)
                    w.Close();
                //throw x;
                Console.WriteLine(x.Message);
                return -1;
            }

        
        }
    }
}
